"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const got_1 = __importDefault(require("got"));
const jsonwebtoken_1 = require("jsonwebtoken");
const node_cache_1 = __importDefault(require("node-cache"));
/**
 */
class AsapRequest {
    /**
     * Constructor
     * @param options
     */
    constructor(options) {
        this.cacheTTL = 60 * 45;
        this.requestTimeoutMs = 3 * 1000;
        this.requestRetryCount = 2;
        this.signingKey = options.signingKey;
        this.asapJwtIss = options.asapJwtIss;
        this.asapJwtAud = options.asapJwtAud;
        this.asapJwtKid = options.asapJwtKid;
        if (options.requestTimeoutMs !== undefined) {
            this.requestTimeoutMs = options.requestTimeoutMs;
        }
        if (options.requestRetryCount !== undefined) {
            this.requestRetryCount = options.requestRetryCount;
        }
        if (options.cacheTTL !== undefined) {
            this.cacheTTL = options.cacheTTL;
        }
        this.asapCache = new node_cache_1.default({ stdTTL: this.cacheTTL }); // TTL of 45 minutes
        this.authToken = this.authToken.bind(this);
        this.getJson = this.getJson.bind(this);
    }
    /**
     */
    authToken() {
        const cachedAuth = this.asapCache.get('asap');
        if (cachedAuth) {
            return cachedAuth;
        }
        const auth = jsonwebtoken_1.sign({}, this.signingKey, {
            issuer: this.asapJwtIss,
            audience: this.asapJwtAud,
            algorithm: 'RS256',
            keyid: this.asapJwtKid,
            expiresIn: 60 * 60 // 1 hour
        });
        this.asapCache.set('asap', auth);
        return auth;
    }
    /**
     * @param ctx
     * @param url
     */
    async getJson(ctx, url) {
        ctx.logger.debug(`Sending GET ${url}`);
        const response = await got_1.default.get(url, {
            headers: {
                Authorization: `Bearer ${this.authToken()}`
            },
            responseType: 'json',
            timeout: this.requestTimeoutMs,
            retry: this.requestRetryCount
        });
        ctx.logger.debug(`GET ${url} response: code ${response.statusCode} and body ${JSON.stringify(response.body)}`);
        return response.body;
    }
    /**
     * @param ctx
     * @param url
     * @param body
     * @param requestOptions
     */
    async postJson(ctx, url, body, requestOptions) {
        ctx.logger.info(`Sending POST ${url}`);
        // by default, got library does not retry post requests
        // we don't need to change this behavior for now
        // https://www.npmjs.com/package/got#retry
        const response = await got_1.default.post(url, {
            headers: {
                Authorization: `Bearer ${this.authToken()}`
            },
            json: body,
            responseType: 'json',
            timeout: requestOptions.requestTimeoutMs ? requestOptions.requestTimeoutMs : this.requestTimeoutMs
        });
        if (response.statusCode === 200) {
            ctx.logger.info(`POST ${url} response: code ${response.statusCode} and body ${JSON.stringify(response.body)}`);
        }
        else {
            ctx.logger.error(`POST ${url} response: code ${response.statusCode} and body ${JSON.stringify(response.body)}`);
        }
        return response.statusCode;
    }
}
exports.default = AsapRequest;
