"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeneratorAuthorization = void 0;
const express_jwt_1 = require("express-jwt");
/**
 * Provider of authorization middlewares
 */
class GeneratorAuthorization {
    /**
     * Constructor
     * @param options
     */
    constructor(options) {
        this.asapFetcher = options.asapFetcher;
        this.protectedApi = options.protectedApi;
        this.systemJwtClaims = options.systemJwtClaims;
        this.jitsiJwtClaims = options.jitsiJwtClaims;
        this.jitsiAuthMiddleware = this.jitsiAuthMiddleware.bind(this);
        this.signalAuthMiddleware = this.signalAuthMiddleware.bind(this);
        this.systemAuthMiddleware = this.systemAuthMiddleware.bind(this);
        this.authorize = this.authorize.bind(this);
    }
    /**
     * Express authorization middleware for jitsi meeting tokens
     * @param req
     * @param res
     * @param next
     */
    jitsiAuthMiddleware(req, res, next) {
        if (req.context) {
            req.context.logger.debug('Trying jitsi authorization');
        }
        this.authorize(req, res, next, this.jitsiJwtClaims);
    }
    /**
     * Express authorization middleware for signaling tokens
     * @param req
     * @param res
     * @param next
     */
    signalAuthMiddleware(req, res, next) {
        if (req.context) {
            req.context.logger.debug('Trying signal authorization');
        }
        // TODO implement authorization
        // next(new UnauthorizedError('revoked_token', { message: 'test error message' }));
        next();
    }
    /**
     * Express authorization middleware for system tokens
     * @param req
     * @param res
     * @param next
     */
    systemAuthMiddleware(req, res, next) {
        if (req.context) {
            req.context.logger.info('Trying system authorization');
        }
        this.authorize(req, res, next, this.systemJwtClaims);
    }
    /**
     * Express-jwt authorization of tokens, taking into consideration the expected jwtClaims.
     * The public key is retrieved from the pre-configured callback associated to the issuer.
     * @param req
     * @param res
     * @param next
     * @param jwtClaims
     * @private
     */
    authorize(req, res, next, jwtClaims) {
        try {
            express_jwt_1.expressjwt({
                secret: this.asapFetcher.pubKeyCallback,
                audience: jwtClaims.asapJwtAcceptedAud,
                issuer: jwtClaims.asapJwtAcceptedHookIss,
                algorithms: ['RS256']
            })
                .unless(() => {
                if (!this.protectedApi) {
                    return true;
                }
                // check for jwt
                return false;
            })
                .apply(this, [req, res, next]);
        }
        catch (err) {
            // if the token has no kid, a TypeError will be thrown. This should be mapped to invalid token
            next(new express_jwt_1.UnauthorizedError('invalid_token', err));
        }
    }
}
exports.GeneratorAuthorization = GeneratorAuthorization;
