"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ASAPPubKeyFetcher = void 0;
const got_1 = __importDefault(require("got"));
const node_cache_1 = __importDefault(require("node-cache"));
const sha256_1 = __importDefault(require("sha256"));
/**
 * Public key fetcher
 */
class ASAPPubKeyFetcher {
    /**
     * Constructor
     * @param issToBaseUrl
     * @param kidPrefixPattern
     * @param ttl
     */
    constructor(issToBaseUrl, kidPrefixPattern, ttl) {
        this.issToBaseUrl = issToBaseUrl;
        this.cache = new node_cache_1.default({ stdTTL: ttl });
        this.kidPrefixPattern = kidPrefixPattern;
        this.pubKeyCallback = this.pubKeyCallback.bind(this);
        this.pubKeyCallbackForJsonWebToken = this.pubKeyCallbackForJsonWebToken.bind(this);
    }
    /**
     * Method for getting the public key
     * @param ctx
     * @param token Jwt token details from jsonwebtoken
     */
    async pubKeyCallbackForJsonWebToken(ctx, token) {
        if (!token.header.kid) {
            throw new Error('kid is required in header');
        }
        let pubKey = this.cache.get(token.header.kid);
        ctx.logger.debug('fetching pub key from key server');
        const payload = token.payload;
        const issuer = payload.iss;
        const baseUrl = this.getPublicKeyUrl(ctx, token.header.kid, this.issToBaseUrl.get(issuer));
        if (!baseUrl) {
            throw new Error('invalid issuer or kid');
        }
        ctx.logger.debug('Fetching pub key from key server');
        try {
            pubKey = await fetchPublicKey(baseUrl, token.header.kid);
            this.cache.set(token.header.kid, pubKey);
        }
        catch (err) {
            ctx.logger.error(`Obtaining asap pub ${err}`, { err });
            throw err;
        }
        return pubKey;
    }
    /**
     * Retrieves the first valid public key URL from the list of mappings
     * @param ctx
     * @param kid
     * @param baseUrlMappings
     * @private
     */
    getPublicKeyUrl(ctx, kid, baseUrlMappings) {
        if (typeof baseUrlMappings === 'undefined') {
            ctx.logger.warn('No public key URL mapping found');
            return null;
        }
        for (const baseUrlMapping of baseUrlMappings) {
            if (!baseUrlMapping.kid) {
                ctx.logger.debug(`Found pub key url mapping: ${baseUrlMapping.baseUrl}`);
                return baseUrlMapping.baseUrl;
            }
            const kidPattern = new RegExp(baseUrlMapping.kid);
            if (kidPattern.test(kid)) {
                if (!baseUrlMapping.appendKidPrefix) {
                    ctx.logger.debug(`Found pub key url mapping by kid pattern: ${baseUrlMapping.baseUrl}`);
                    return baseUrlMapping.baseUrl;
                }
                if (this.kidPrefixPattern.test(kid)) {
                    const baseUrl = baseUrlMapping.baseUrl.concat('/').concat(this.kidPrefixPattern.exec(kid)[1]);
                    ctx.logger.debug(`Found pub key url mapping by kid pattern and suffix: ${baseUrl}`);
                    return baseUrl;
                }
            }
        }
        return null;
    }
    /**
     * Method for getting the public key, using a signature
     * specific to express-jwt
     * @param req
     * @param token JWT from jsonwebtoken
     */
    async pubKeyCallback(req, token) {
        return this.pubKeyCallbackForJsonWebToken(req.context, token);
    }
}
exports.ASAPPubKeyFetcher = ASAPPubKeyFetcher;
/**
 * Method for getting the public key from server
 * @param baseUrl
 * @param kid
 */
async function fetchPublicKey(baseUrl, kid) {
    const hashedKid = sha256_1.default(kid);
    const reqUrl = `${baseUrl}/${hashedKid}.pem`;
    const response = await got_1.default(reqUrl);
    return response.body;
}
